@@ -45,8 +45,6 @@
 #include "virfdstream.h"
 #include "virpidfile.h"
 #include "configmake.h"
-#include "virsecret.h"
-#include "virstring.h"
 #include "viraccessapicheck.h"
 #include "storage_util.h"
 #include "virutil.h"
@@ -244,6 +242,7 @@ storageDriverAutostart(void)
 static int
 storageStateInitialize(bool privileged,
                        const char *root,
+                       bool monolithic G_GNUC_UNUSED,
                        virStateInhibitCallback callback G_GNUC_UNUSED,
                        void *opaque G_GNUC_UNUSED)
 {
